"""
Módulo de operaciones de datos de mercado para pyRofex-To-Excel.

Este módulo maneja todas las operaciones de datos de mercado incluyendo cliente API,
conexiones WebSocket y procesamiento de datos.
"""

from .api_client import pyRofexClient
from .data_processor import DataProcessor
from .websocket_handler import WebSocketHandler

__all__ = [
    'pyRofexClient',
    'WebSocketHandler', 
    'DataProcessor'
]